<?php
#
# Script         backup.php
# Copyright      2013 Denny Schlesinger
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# Autor          Denny Schlesinger
#
# $Id: $
#
############################################################################

interface Backup {

    // creates the backup file (filename.sql)
    public function __construct($username, 
                                $password, 
                                $database, 
                                $tables = "", 
                                $host = "localhost", 
                                $path2files = "");
                                
    // downloads the backup file
    // as filename.sql
    // or compressed as filename.zip (default)
    public function download($compression = "zip");

}

if (!function_exists('__autoload')) {
    function __autoload($class_name) { 
        require_once $class_name . '.php';
    }
}

class MySQLbackup implements Backup {

    // database
    public $host = "localhost";
    public $username;
    public $password;
    public $database;
    public $tables;
    
    //file
    public $path2files;
    public $filename;
    
    public function __construct($username, $password, $database, $tables="", $host="", $path2files="") {
        if(!isset($username) OR
           !isset($password) OR
           !isset($database)) {
               trigger_error("Missing construct parameters", E_USER_ERROR);
        } else {
            $this->username = $username;
            $this->password = $password;
            $this->database = $database;
            $this->tables   = $tables;
        }
        
        if($host != "") {
            $this->host = $host;
        }
        
        if($path2files != "") {
            $this->path2files = $path2files;
        } else {
            // get root directory
            $dir = explode('/',getcwd());
            $this->path2files = "/{$dir[1]}/{$dir[2]}/";
        }
        
        // connect to database
        $mysqli = @new mysqli($this->host, $this->username, $this->password, $this->database);
        if (mysqli_connect_error()) {
            trigger_error('Connect Error (' . mysqli_connect_errno() . ') '
                    . mysqli_connect_error(), E_USER_ERROR);
        }

        // get table names
        $tablesIn = "Tables_in_{$this->database}";
        $tableNames = array();
        if (!$result = $mysqli->query("SHOW TABLES")) {
            trigger_error($mysqli->error, E_USER_ERROR);
        }
        while ($row = $result->fetch_array(MYSQLI_ASSOC)) {
            $tableNames[] = $row[$tablesIn];
        }
    
        $result->close();
        if (count($tableNames) == 0) {
            trigger_error("No tables found in database", E_USER_ERROR);
        }
  
        // check table name input
        if ($this->tables == "") {
            $this->tables = $tableNames;
        } elseif (is_string($this->tables)) {
            $this->tables = explode(',', $this->tables);
            foreach ($this->tables AS $key=>$value) {
                if (!in_array ($value, $tableNames)) {
                    trigger_error("Unknown table in database", E_USER_ERROR);
                }
                $this->tables[$key] = trim($value);
            }
        } elseif (!is_array($this->tables)) {
            trigger_error("Invalid table name type", E_USER_ERROR);
        } elseif (count($this->tables) == 0) {
            $this->tables = $tableNames;
        } else {
            foreach ($this->tables AS $value) {
                if (!in_array ($value, $tableNames)) {
                    trigger_error("Unknown table in database", E_USER_ERROR);
                }
            }
        }
  
        //cycle through tables
        $return = "";
        foreach($this->tables as $table) {
            $return.= "DROP TABLE IF EXISTS `{$table}`;";
            if (!$result = $mysqli->query("SHOW CREATE TABLE {$table}")) {
                trigger_error($mysqli->error, E_USER_ERROR);
            }
            $create = $result->fetch_array(MYSQLI_ASSOC);
            $return.= "\n".$create['Create Table'].";\n\n";
    
            // get field names
            if (!$result = $mysqli->query("SHOW columns FROM `{$table}`")) {
                trigger_error($mysqli->error, E_USER_ERROR);
            }
            $names = array();
            while ($row = $result->fetch_array(MYSQLI_ASSOC)) {
                $names[] = "`{$row['Field']}`";
            }
            
            // get field data
            if (!$result = $mysqli->query("SELECT * FROM `{$table}`")) {
                trigger_error($mysqli->error, E_USER_ERROR);
            }
            while ($row = $result->fetch_row()) {
                $return.= "INSERT INTO `{$table}` VALUES(";
                foreach ($row AS $value) {
                    $value = addslashes($value);
                    $value = ereg_replace("\n","\\n",$value);
                    if (!is_numeric($value)) { $value = "'{$value}'";}
                    $return .= "{$value}, ";
                }
                $return = substr ($return, 0, -2);
                $return .= ");\n";
            }
    
            if (!$result = $mysqli->query("SELECT * FROM {$table}")) {
                trigger_error($mysqli->error, E_USER_ERROR);
            }
            
            $return.="\n\n\n";
        
        } // foreach($this->tables as $table)
    
        if(count($this->tables) == 1) {
            $this->filename = "{$this->path2files}{$this->tables[0]}.sql";
        } else {
            $this->filename .= "{$this->path2files}{$this->database}.sql";
        }
        
        if (@file_put_contents ($this->filename, $return) === FALSE) {
            trigger_error("Cannot write to file {$this->filename}", E_USER_ERROR);
        }

    } // function __construct()
               
    public function download($compression = "zip") {
    
        switch($compression) {
        
        case "zip":
        
            $sqlname = array_pop(explode('/', $this->filename));
            $zipname = substr_replace($this->filename, '.zip', -4);
            $downloadname = array_pop(explode('/', $zipname));
            
            // delete old file
            if(file_exists($zipname)) {
                @unlink($zipname);
            }
            
            $zip = new ZipArchive();
            if ($zip->open($zipname, ZIPARCHIVE::CREATE) !== TRUE) {
                trigger_error("Cannot open <$zipname>\n", E_USER_ERROR);
            }
            //$zip->addFile($this->filename);
            $zip->addFromString ($sqlname, file_get_contents($this->filename));
            $zip->close();
            
            // push to download the zip
            header('Content-type: application/zip');
            header('Content-Disposition: attachment; filename="'.$downloadname.'"');
            readfile($zipname);
            
            break;

        case "sql":
        
            $downloadname = array_pop(explode('/', $this->filename));
        
            // push to download the file
            header('Content-type: application/octet-stream');
            header('Content-Disposition: attachment; filename="'.$downloadname.'"');
            readfile($this->filename);
            
            break;
        
        default:
        
            trigger_error("Invalid compression type '{$compression}')", E_USER_ERROR);
            
        } // switch($compression)
        
    } // function download()

}

?>
